@@ -1,6 +1,6 @@
 /****************************************************************************/
 // Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2007-2018 German Aerospace Center (DLR) and others.
+// Copyright (C) 2007-2019 German Aerospace Center (DLR) and others.
 // This program and the accompanying materials
 // are made available under the terms of the Eclipse Public License v2.0
 // which accompanies this distribution, and is available at
@@ -14,7 +14,6 @@
 /// @author  Christoph Sommer
 /// @author  Jakob Erdmann
 /// @date    Tue, 04 Dec 2007
-/// @version $Id$
 ///
 // A device that performs vehicle rerouting based on current edge speeds
 /****************************************************************************/
@@ -37,10 +36,10 @@
 #include <utils/common/StaticCommand.h>
 #include <utils/common/StringUtils.h>
 #include <utils/xml/SUMOSAXAttributes.h>
-#include <utils/vehicle/DijkstraRouter.h>
-#include <utils/vehicle/AStarRouter.h>
-#include <utils/vehicle/CHRouter.h>
-#include <utils/vehicle/CHRouterWrapper.h>
+#include <utils/router/DijkstraRouter.h>
+#include <utils/router/AStarRouter.h>
+#include <utils/router/CHRouter.h>
+#include <utils/router/CHRouterWrapper.h>
 
 
 // ===========================================================================
@@ -56,20 +55,16 @@ SUMOTime MSRoutingEngine::myAdaptationInterval = -1;
 SUMOTime MSRoutingEngine::myLastAdaptation = -1;
 bool MSRoutingEngine::myWithTaz;
 SUMOAbstractRouter<MSEdge, SUMOVehicle>* MSRoutingEngine::myRouter = nullptr;
-AStarRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >* MSRoutingEngine::myRouterWithProhibited = nullptr;
-double MSRoutingEngine::myRandomizeWeightsFactor = 0;
 std::map<std::pair<const MSEdge*, const MSEdge*>, const MSRoute*> MSRoutingEngine::myCachedRoutes;
+SUMOAbstractRouter<MSEdge, SUMOVehicle>::Operation MSRoutingEngine::myEffortFunc = &MSRoutingEngine::getEffort;
 #ifdef HAVE_FOX
-FXWorkerThread::Pool MSRoutingEngine::myThreadPool;
+FXMutex MSRoutingEngine::myRouteCacheMutex;
 #endif
 
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
-// ---------------------------------------------------------------------------
-// static initialisation methods
-// ---------------------------------------------------------------------------
 void
 MSRoutingEngine::initWeightUpdate() {
     if (myAdaptationInterval == -1) {
@@ -93,14 +88,14 @@ MSRoutingEngine::initWeightUpdate() {
     }
 }
 
-// ---------------------------------------------------------------------------
-// MSRoutingEngine-methods
-// ---------------------------------------------------------------------------
+
 void
 MSRoutingEngine::initEdgeWeights() {
     if (myEdgeSpeeds.empty()) {
         const OptionsCont& oc = OptionsCont::getOptions();
-        myAdaptationSteps = oc.getInt("device.rerouting.adaptation-steps");
+        if (myAdaptationWeight == 0 || !oc.isDefault("device.rerouting.adaptation-steps")) {
+            myAdaptationSteps = oc.getInt("device.rerouting.adaptation-steps");
+        }
         const bool useLoaded = oc.getBool("device.rerouting.init-with-loaded-weights");
         const double currentSecond = SIMTIME;
         for (const MSEdge* const edge : MSNet::getInstance()->getEdgeControl().getEdges()) {
@@ -120,7 +115,6 @@ MSRoutingEngine::initEdgeWeights() {
             }
         }
         myLastAdaptation = MSNet::getInstance()->getCurrentTimeStep();
-        myRandomizeWeightsFactor = oc.getFloat("weights.random-factor");
     }
 }
 
@@ -129,19 +123,29 @@ double
 MSRoutingEngine::getEffort(const MSEdge* const e, const SUMOVehicle* const v, double) {
     const int id = e->getNumericalID();
     if (id < (int)myEdgeSpeeds.size()) {
-        double effort = MAX2(e->getLength() / MAX2(myEdgeSpeeds[id], NUMERICAL_EPS), e->getMinimumTravelTime(v));
-        if (myRandomizeWeightsFactor != 1.) {
-            effort *= RandHelper::rand(1., myRandomizeWeightsFactor);
-        }
-        return effort;
+        return MAX2(e->getLength() / MAX2(myEdgeSpeeds[id], NUMERICAL_EPS), e->getMinimumTravelTime(v));
     }
     return 0.;
 }
 
 
+double
+MSRoutingEngine::getEffortExtra(const MSEdge* const e, const SUMOVehicle* const v, double t) {
+    if (e->getBidiEdge() != nullptr && !e->getBidiEdge()->getLanes()[0]->isEmpty()) {
+        // using std::numeric_limits<double>::max() causing router warnings
+        return e->getLength() / NUMERICAL_EPS;
+    }
+    double effort = getEffort(e, v, t);
+    if (gWeightsRandomFactor != 1.) {
+        effort *= RandHelper::rand(1., gWeightsRandomFactor);
+    }
+    return effort;
+}
+
+
 double
 MSRoutingEngine::getAssumedSpeed(const MSEdge* edge) {
-    return edge->getLength() / getEffort(edge, nullptr, 0);
+    return edge->getLength() / myEffortFunc(edge, nullptr, 0);
 }
 
 
@@ -188,11 +192,10 @@ MSRoutingEngine::adaptEdgeEfforts(SUMOTime currentTime) {
         dev.writeAttr(SUMO_ATTR_ID, "device.rerouting");
         dev.writeAttr(SUMO_ATTR_BEGIN, STEPS2TIME(currentTime));
         dev.writeAttr(SUMO_ATTR_END, STEPS2TIME(currentTime + myAdaptationInterval));
-        for (MSEdgeVector::const_iterator i = edges.begin(); i != edges.end(); ++i) {
-            const int id = (*i)->getNumericalID();
+        for (const MSEdge* e : edges) {
             dev.openTag(SUMO_TAG_EDGE);
-            dev.writeAttr(SUMO_ATTR_ID, (*i)->getID());
-            dev.writeAttr("traveltime", (*i)->getLength() / myEdgeSpeeds[id]);
+            dev.writeAttr(SUMO_ATTR_ID, e->getID());
+            dev.writeAttr("traveltime", myEffortFunc(e, nullptr, STEPS2TIME(currentTime)));
             dev.closeTag();
         }
         dev.closeTag();
@@ -212,92 +215,85 @@ MSRoutingEngine::getCachedRoute(const std::pair<const MSEdge*, const MSEdge*>& k
 
 
 void
-MSRoutingEngine::reroute(SUMOVehicle& vehicle, const SUMOTime currentTime, const bool onInit) {
+MSRoutingEngine::initRouter(SUMOVehicle* vehicle) {
+    OptionsCont& oc = OptionsCont::getOptions();
+    const std::string routingAlgorithm = oc.getString("routing-algorithm");
+    myEffortFunc = (gWeightsRandomFactor != 1 || MSNet::getInstance()->hasBidiEdges()
+                    ? &MSRoutingEngine::getEffortExtra
+                    : &MSRoutingEngine::getEffort);
+    if (routingAlgorithm == "dijkstra") {
+        myRouter = new DijkstraRouter<MSEdge, SUMOVehicle>(MSEdge::getAllEdges(), true, myEffortFunc, nullptr, false, nullptr, true);
+    } else if (routingAlgorithm == "astar") {
+        typedef AStarRouter<MSEdge, SUMOVehicle> AStar;
+        std::shared_ptr<const AStar::LookupTable> lookup = nullptr;
+        if (oc.isSet("astar.all-distances")) {
+            lookup = std::make_shared<const AStar::FLT>(oc.getString("astar.all-distances"), (int)MSEdge::getAllEdges().size());
+        } else if (oc.isSet("astar.landmark-distances") && vehicle != nullptr) {
+            const double speedFactor = vehicle->getChosenSpeedFactor();
+            // we need an exemplary vehicle with speedFactor 1
+            vehicle->setChosenSpeedFactor(1);
+            CHRouterWrapper<MSEdge, SUMOVehicle> router(
+                MSEdge::getAllEdges(), true, &MSNet::getTravelTime,
+                string2time(oc.getString("begin")), string2time(oc.getString("end")), std::numeric_limits<int>::max(), 1);
+            lookup = std::make_shared<const AStar::LMLT>(oc.getString("astar.landmark-distances"), MSEdge::getAllEdges(), &router, vehicle, "", oc.getInt("device.rerouting.threads"));
+            vehicle->setChosenSpeedFactor(speedFactor);
+        }
+        myRouter = new AStar(MSEdge::getAllEdges(), true, myEffortFunc, lookup, true);
+    } else if (routingAlgorithm == "CH") {
+        const SUMOTime weightPeriod = myAdaptationInterval > 0 ? myAdaptationInterval : std::numeric_limits<int>::max();
+        myRouter = new CHRouter<MSEdge, SUMOVehicle>(
+            MSEdge::getAllEdges(), true, myEffortFunc, vehicle->getVClass(), weightPeriod, true, false);
+    } else if (routingAlgorithm == "CHWrapper") {
+        const SUMOTime weightPeriod = myAdaptationInterval > 0 ? myAdaptationInterval : std::numeric_limits<int>::max();
+        myRouter = new CHRouterWrapper<MSEdge, SUMOVehicle>(
+            MSEdge::getAllEdges(), true, myEffortFunc,
+            string2time(oc.getString("begin")), string2time(oc.getString("end")), weightPeriod, oc.getInt("device.rerouting.threads"));
+    } else {
+        throw ProcessError("Unknown routing algorithm '" + routingAlgorithm + "'!");
+    }
 #ifdef HAVE_FOX
-    const bool needThread = (myRouter == nullptr && myThreadPool.isFull());
-#else
-    const bool needThread = true;
+    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        const std::vector<FXWorkerThread*>& threads = threadPool.getWorkers();
+        if (static_cast<MSEdgeControl::WorkerThread*>(threads.front())->setRouter(myRouter)) {
+            for (std::vector<FXWorkerThread*>::const_iterator t = threads.begin() + 1; t != threads.end(); ++t) {
+                static_cast<MSEdgeControl::WorkerThread*>(*t)->setRouter(myRouter->clone());
+            }
+        }
+    }
 #endif
-    if (needThread && myRouter == nullptr) {
-        OptionsCont& oc = OptionsCont::getOptions();
-        const std::string routingAlgorithm = oc.getString("routing-algorithm");
-        const bool mayHaveRestrictions = MSNet::getInstance()->hasPermissions() || oc.getInt("remote-port") != 0;
-        if (routingAlgorithm == "dijkstra") {
-            if (mayHaveRestrictions) {
-                myRouter = new DijkstraRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-                    MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort);
-            } else {
-                myRouter = new DijkstraRouter<MSEdge, SUMOVehicle, SUMOAbstractRouter<MSEdge, SUMOVehicle> >(
-                    MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort);
-            }
-        } else if (routingAlgorithm == "astar") {
-            if (mayHaveRestrictions) {
-                typedef AStarRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> > AStar;
-                std::shared_ptr<const AStar::LookupTable> lookup;
-                if (oc.isSet("astar.all-distances")) {
-                    lookup = std::make_shared<const AStar::FLT>(oc.getString("astar.all-distances"), (int)MSEdge::getAllEdges().size());
-                } else if (oc.isSet("astar.landmark-distances")) {
-                    const double speedFactor = vehicle.getChosenSpeedFactor();
-                    // we need an exemplary vehicle with speedFactor 1
-                    vehicle.setChosenSpeedFactor(1);
-                    CHRouterWrapper<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> > router(
-                        MSEdge::getAllEdges(), true, &MSNet::getTravelTime,
-                        string2time(oc.getString("begin")), string2time(oc.getString("end")), std::numeric_limits<int>::max(), 1);
-                    lookup = std::make_shared<const AStar::LMLT>(oc.getString("astar.landmark-distances"), MSEdge::getAllEdges(), &router, &vehicle, "", oc.getInt("device.rerouting.threads"));
-                    vehicle.setChosenSpeedFactor(speedFactor);
-                }
-                myRouter = new AStar(MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort, lookup);
-            } else {
-                typedef AStarRouter<MSEdge, SUMOVehicle, SUMOAbstractRouter<MSEdge, SUMOVehicle> > AStar;
-                std::shared_ptr<const AStar::LookupTable> lookup;
-                if (oc.isSet("astar.all-distances")) {
-                    lookup = std::shared_ptr<const AStar::LookupTable> ( new AStar::FLT(oc.getString("astar.all-distances"), (int)MSEdge::getAllEdges().size()));
-                } else if (oc.isSet("astar.landmark-distances")) {
-                    const double speedFactor = vehicle.getChosenSpeedFactor();
-                    // we need an exemplary vehicle with speedFactor 1
-                    vehicle.setChosenSpeedFactor(1);
-                    CHRouterWrapper<MSEdge, SUMOVehicle, SUMOAbstractRouter<MSEdge, SUMOVehicle> > router(
-                        MSEdge::getAllEdges(), true, &MSNet::getTravelTime,
-                        string2time(oc.getString("begin")), string2time(oc.getString("end")), std::numeric_limits<int>::max(), 1);
-                    lookup = std::make_shared<const AStar::LMLT>(oc.getString("astar.landmark-distances"), MSEdge::getAllEdges(), &router, &vehicle, "", oc.getInt("device.rerouting.threads"));
-                    vehicle.setChosenSpeedFactor(speedFactor);
-                }
-                myRouter = new AStar(MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort, lookup);
-            }
-        } else if (routingAlgorithm == "CH") {
-            const SUMOTime weightPeriod = myAdaptationInterval > 0 ? myAdaptationInterval : std::numeric_limits<int>::max();
-            if (mayHaveRestrictions) {
-                myRouter = new CHRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-                    MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort, vehicle.getVClass(), weightPeriod, true);
-            } else {
-                myRouter = new CHRouter<MSEdge, SUMOVehicle, SUMOAbstractRouter<MSEdge, SUMOVehicle> >(
-                    MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort, vehicle.getVClass(), weightPeriod, false);
-            }
-        } else if (routingAlgorithm == "CHWrapper") {
-            const SUMOTime weightPeriod = myAdaptationInterval > 0 ? myAdaptationInterval : std::numeric_limits<int>::max();
-            myRouter = new CHRouterWrapper<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-                MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort,
-                string2time(oc.getString("begin")), string2time(oc.getString("end")), weightPeriod, oc.getInt("device.rerouting.threads"));
-        } else {
-            throw ProcessError("Unknown routing algorithm '" + routingAlgorithm + "'!");
-        }
+}
+
+
+void
+MSRoutingEngine::reroute(SUMOVehicle& vehicle, const SUMOTime currentTime, const std::string& info,
+                         const bool onInit, const bool silent, const MSEdgeVector& prohibited) {
+    if (myRouter == nullptr) {
+        initRouter(&vehicle);
     }
 #ifdef HAVE_FOX
-    if (needThread) {
-        const int numThreads = OptionsCont::getOptions().getInt("device.rerouting.threads");
-        if (myThreadPool.size() < numThreads) {
-            new WorkerThread(myThreadPool, myRouter);
-        }
-        if (myThreadPool.size() < numThreads) {
-            myRouter = nullptr;
-        }
-    }
-    if (myThreadPool.size() > 0) {
-        myThreadPool.add(new RoutingTask(vehicle, currentTime, onInit));
+    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        threadPool.add(new RoutingTask(vehicle, currentTime, info, onInit, silent, prohibited));
         return;
     }
 #endif
-    vehicle.reroute(currentTime, "device.rerouting", *myRouter, onInit, myWithTaz);
+    if (!prohibited.empty()) {
+        myRouter->prohibit(prohibited);
+    }
+    try {
+        vehicle.reroute(currentTime, info, *myRouter, onInit, myWithTaz, silent);
+    } catch (ProcessError&) {
+        if (!silent) {
+            if (!prohibited.empty()) {
+                myRouter->prohibit(MSEdgeVector());
+            }
+            throw;
+        }
+    }
+    if (!prohibited.empty()) {
+        myRouter->prohibit(MSEdgeVector());
+    }
 }
 
 
@@ -308,22 +304,27 @@ MSRoutingEngine::setEdgeTravelTime(const MSEdge* const edge, const double travel
 
 
 SUMOAbstractRouter<MSEdge, SUMOVehicle>&
-MSRoutingEngine::getRouterTT(const MSEdgeVector& prohibited) {
-    if (myRouterWithProhibited == nullptr) {
+MSRoutingEngine::getRouterTT(const int rngIndex, const MSEdgeVector& prohibited) {
+    if (myRouter == nullptr) {
         initWeightUpdate();
         initEdgeWeights();
-        myRouterWithProhibited = new AStarRouter<MSEdge, SUMOVehicle, SUMOAbstractRouterPermissions<MSEdge, SUMOVehicle> >(
-            MSEdge::getAllEdges(), true, &MSRoutingEngine::getEffort);
+        initRouter();
     }
-    myRouterWithProhibited->prohibit(prohibited);
-    return *myRouterWithProhibited;
+#ifdef HAVE_FOX
+    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        auto& router = static_cast<MSEdgeControl::WorkerThread*>(threadPool.getWorkers()[rngIndex % MSGlobals::gNumThreads])->getRouter();
+        router.prohibit(prohibited);
+        return router;
+    }
+#endif
+    myRouter->prohibit(prohibited);
+    return *myRouter;
 }
 
 
 void
 MSRoutingEngine::cleanup() {
-    delete myRouterWithProhibited;
-    myRouterWithProhibited = nullptr;
     myAdaptationInterval = -1; // responsible for triggering initEdgeWeights
     myPastEdgeSpeeds.clear();
     myEdgeSpeeds.clear();
@@ -334,10 +335,7 @@ MSRoutingEngine::cleanup() {
     myCachedRoutes.clear();
     myAdaptationStepsIndex = 0;
 #ifdef HAVE_FOX
-    if (myThreadPool.size() > 0) {
-        // we cannot wait for the static destructor to do the cleanup
-        // because the output devices are gone by then
-        myThreadPool.clear();
+    if (MSGlobals::gNumThreads > 1) {
         // router deletion is done in thread destructor
         myRouter = nullptr;
         return;
@@ -351,8 +349,9 @@ MSRoutingEngine::cleanup() {
 #ifdef HAVE_FOX
 void
 MSRoutingEngine::waitForAll() {
-    if (myThreadPool.size() > 0) {
-        myThreadPool.waitAll();
+    FXWorkerThread::Pool& threadPool = MSNet::getInstance()->getEdgeControl().getThreadPool();
+    if (threadPool.size() > 0) {
+        threadPool.waitAll();
     }
 }
 
@@ -362,21 +361,35 @@ MSRoutingEngine::waitForAll() {
 // ---------------------------------------------------------------------------
 void
 MSRoutingEngine::RoutingTask::run(FXWorkerThread* context) {
-    myVehicle.reroute(myTime, "device.rerouting", static_cast<WorkerThread*>(context)->getRouter(), myOnInit, myWithTaz);
+    SUMOAbstractRouter<MSEdge, SUMOVehicle>& router = static_cast<MSEdgeControl::WorkerThread*>(context)->getRouter();
+    if (!myProhibited.empty()) {
+        router.prohibit(myProhibited);
+    }
+    try {
+        myVehicle.reroute(myTime, myInfo, router, myOnInit, myWithTaz, mySilent);
+    } catch (ProcessError&) {
+        if (!mySilent) {
+            if (!myProhibited.empty()) {
+                router.prohibit(MSEdgeVector());
+            }
+            throw;
+        }
+    }
+    if (!myProhibited.empty()) {
+        router.prohibit(MSEdgeVector());
+    }
     const MSEdge* source = *myVehicle.getRoute().begin();
     const MSEdge* dest = myVehicle.getRoute().getLastEdge();
     if (source->isTazConnector() && dest->isTazConnector()) {
         const std::pair<const MSEdge*, const MSEdge*> key = std::make_pair(source, dest);
-        lock();
+        FXMutexLock lock(myRouteCacheMutex);
         if (MSRoutingEngine::myCachedRoutes.find(key) == MSRoutingEngine::myCachedRoutes.end()) {
             MSRoutingEngine::myCachedRoutes[key] = &myVehicle.getRoute();
             myVehicle.getRoute().addReference();
         }
-        unlock();
     }
 }
 #endif
 
 
 /****************************************************************************/
-
