@@ -1,35 +1,27 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
+// Copyright (C) 2001-2019 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials
+// are made available under the terms of the Eclipse Public License v2.0
+// which accompanies this distribution, and is available at
+// http://www.eclipse.org/legal/epl-v20.html
+// SPDX-License-Identifier: EPL-2.0
+/****************************************************************************/
 /// @file    PCLoaderVisum.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Jakob Erdmann
 /// @author  Christoph Sommer
 /// @author  Michael Behrisch
 /// @date    Thu, 02.11.2006
-/// @version $Id: PCLoaderVisum.cpp 20801 2016-05-28 05:31:30Z behrisch $
 ///
 // A reader of pois and polygons stored in VISUM-format
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2001-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
 
 
 // ===========================================================================
 // included modules
 // ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <string>
 #include <map>
@@ -38,7 +30,7 @@
 #include <utils/common/UtilExceptions.h>
 #include <utils/common/MsgHandler.h>
 #include <utils/common/StringUtils.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/common/ToString.h>
 #include <utils/common/FileHelpers.h>
 #include <utils/options/OptionsCont.h>
@@ -54,10 +46,6 @@
 #include <utils/geom/GeoConvHelper.h>
 #include <utils/importio/NamedColumnsParser.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
@@ -102,9 +90,9 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
         // read items
         if (what == "$PUNKT") {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("ID").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int id = StringUtils::toLong(lineParser.get("ID"));
+            double x = StringUtils::toDouble(lineParser.get("XKOORD"));
+            double y = StringUtils::toDouble(lineParser.get("YKOORD"));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for point '" + toString(id) + "'.");
@@ -113,9 +101,9 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             continue;
         } else if (what == "$KANTE") {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("ID").c_str());
-            long long int fromID = TplConvert::_2long(lineParser.get("VONPUNKTID").c_str());
-            long long int toID = TplConvert::_2long(lineParser.get("NACHPUNKTID").c_str());
+            long long int id = StringUtils::toLong(lineParser.get("ID"));
+            long long int fromID = StringUtils::toLong(lineParser.get("VONPUNKTID"));
+            long long int toID = StringUtils::toLong(lineParser.get("NACHPUNKTID"));
             PositionVector vec;
             vec.push_back(punkte[fromID]);
             vec.push_back(punkte[toID]);
@@ -123,10 +111,10 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             continue;
         } else if (what == "$ZWISCHENPUNKT") {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("KANTEID").c_str());
-            int index = TplConvert::_2int(lineParser.get("INDEX").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int id = StringUtils::toLong(lineParser.get("KANTEID"));
+            int index = StringUtils::toInt(lineParser.get("INDEX"));
+            double x = StringUtils::toDouble(lineParser.get("XKOORD"));
+            double y = StringUtils::toDouble(lineParser.get("YKOORD"));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for edge '" + toString(id) + "'.");
@@ -135,11 +123,11 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             continue;
         } else if (what == "$TEILFLAECHENELEMENT") {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("TFLAECHEID").c_str());
-            //int index = TplConvert::_2int(lineParser.get("INDEX").c_str());
+            long long int id = StringUtils::toLong(lineParser.get("TFLAECHEID"));
+            //int index = StringUtils::toInt(lineParser.get("INDEX"));
             //index = 0; /// hmmmm - assume it's sorted...
-            long long int kid = TplConvert::_2long(lineParser.get("KANTEID").c_str());
-            int dir = TplConvert::_2int(lineParser.get("RICHTUNG").c_str());
+            long long int kid = StringUtils::toLong(lineParser.get("KANTEID"));
+            int dir = StringUtils::toInt(lineParser.get("RICHTUNG"));
             if (teilflaechen.find(id) == teilflaechen.end()) {
                 teilflaechen[id] = PositionVector();
             }
@@ -155,8 +143,8 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             continue;
         } else if (what == "$FLAECHENELEMENT") {
             lineParser.parseLine(line);
-            long long int id = TplConvert::_2long(lineParser.get("FLAECHEID").c_str());
-            long long int tid = TplConvert::_2long(lineParser.get("TFLAECHEID").c_str());
+            long long int id = StringUtils::toLong(lineParser.get("FLAECHEID"));
+            long long int tid = StringUtils::toLong(lineParser.get("TFLAECHEID"));
             flaechenelemente[id] = tid;
             continue;
         }
@@ -221,12 +209,12 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             // parse the poi
             // $POI:Nr;CATID;CODE;NAME;Kommentar;XKoord;YKoord;
             lineParser.parseLine(line);
-            long long int idL = TplConvert::_2long(lineParser.get("Nr").c_str());
+            long long int idL = StringUtils::toLong(lineParser.get("Nr"));
             std::string id = toString(idL);
             std::string catid = lineParser.get("CATID");
             // process read values
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKoord").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKoord").c_str());
+            double x = StringUtils::toDouble(lineParser.get("XKoord"));
+            double y = StringUtils::toDouble(lineParser.get("YKoord"));
             Position pos(x, y);
             if (!geoConvHelper.x2cartesian(pos)) {
                 WRITE_WARNING("Unable to project coordinates for POI '" + id + "'.");
@@ -234,7 +222,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             std::string type = typemap[catid];
             // patch the values
             bool discard = oc.getBool("discard");
-            SUMOReal layer = oc.getFloat("layer");
+            double layer = oc.getFloat("layer");
             RGBColor color;
             if (tm.has(type)) {
                 const PCTypeMap::TypeDef& def = tm.get(type);
@@ -249,7 +237,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
                 color = c;
             }
             if (!discard) {
-                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, layer);
+                PointOfInterest* poi = new PointOfInterest(id, type, color, pos, false, "", 0, 0, layer);
                 toFill.add(poi);
             }
         }
@@ -262,7 +250,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             if (!first && lastID != id) {
                 // we have parsed a polygon completely
                 RGBColor color;
-                SUMOReal layer = oc.getFloat("layer");
+                double layer = oc.getFloat("layer");
                 bool discard = oc.getBool("discard");
                 if (tm.has(polyType)) {
                     const PCTypeMap::TypeDef& def = tm.get(polyType);
@@ -277,7 +265,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
                     color = c;
                 }
                 if (!discard) {
-                    SUMO::Polygon* poly = new SUMO::Polygon(id, type, color, vec, false, layer);
+                    SUMOPolygon* poly = new SUMOPolygon(id, type, color, vec, false, false, 1, layer);
                     toFill.add(poly);
                 }
                 vec.clear();
@@ -288,7 +276,7 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             std::string index = st.next();
             std::string xpos = st.next();
             std::string ypos = st.next();
-            Position pos2D((SUMOReal) atof(xpos.c_str()), (SUMOReal) atof(ypos.c_str()));
+            Position pos2D((double) atof(xpos.c_str()), (double) atof(ypos.c_str()));
             if (!geoConvHelper.x2cartesian(pos2D)) {
                 WRITE_WARNING("Unable to project coordinates for polygon '" + id + "'.");
             }
@@ -299,15 +287,15 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
         if (parsingDistrictsDirectly) {
             //$BEZIRK:NR	CODE	NAME	TYPNR	XKOORD	YKOORD	FLAECHEID	BEZART	IVANTEIL_Q	IVANTEIL_Z	OEVANTEIL	METHODEANBANTEILE	ZWERT1	ZWERT2	ZWERT3	ISTINAUSWAHL	OBEZNR	NOM_COM	COD_COM
             lineParser.parseLine(line);
-            long long int idL = TplConvert::_2long(lineParser.get("NR").c_str());
+            long long int idL = StringUtils::toLong(lineParser.get("NR"));
             std::string id = toString(idL);
-            long long int area = TplConvert::_2long(lineParser.get("FLAECHEID").c_str());
-            SUMOReal x = TplConvert::_2SUMOReal(lineParser.get("XKOORD").c_str());
-            SUMOReal y = TplConvert::_2SUMOReal(lineParser.get("YKOORD").c_str());
+            long long int area = StringUtils::toLong(lineParser.get("FLAECHEID"));
+            double x = StringUtils::toDouble(lineParser.get("XKOORD"));
+            double y = StringUtils::toDouble(lineParser.get("YKOORD"));
             // patch the values
             std::string type = "district";
             bool discard = oc.getBool("discard");
-            SUMOReal layer = oc.getFloat("layer");
+            double layer = oc.getFloat("layer");
             RGBColor color;
             if (tm.has(type)) {
                 const PCTypeMap::TypeDef& def = tm.get(type);
@@ -323,14 +311,14 @@ PCLoaderVisum::load(const std::string& file, OptionsCont& oc, PCPolyContainer& t
             }
             if (!discard) {
                 if (teilflaechen[flaechenelemente[area]].size() > 0) {
-                    SUMO::Polygon* poly = new SUMO::Polygon(id, type, color, teilflaechen[flaechenelemente[area]], false, layer);
+                    SUMOPolygon* poly = new SUMOPolygon(id, type, color, teilflaechen[flaechenelemente[area]], false, false, 1, layer);
                     toFill.add(poly);
                 } else {
                     Position pos(x, y);
                     if (!geoConvHelper.x2cartesian(pos)) {
                         WRITE_WARNING("Unable to project coordinates for POI '" + id + "'.");
                     }
-                    PointOfInterest* poi = new PointOfInterest(id, type, color, pos, layer);
+                    PointOfInterest* poi = new PointOfInterest(id, type, color, pos, "", nullptr, 0, layer);
                     toFill.add(poi);
                 }
             }
